import json
from urllib2 import urlopen
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.core.management.base import CommandError, BaseCommand
from currencies.models import Currency

CURRENCY_API_URL = "http://openexchangerates.org/currencies.json"


class Command(BaseCommand):
	help = "Create all missing currencies defined on openexchangerates.org. A list of ISO 4217 codes may be supplied to limit the currencies initialised."
	args = "<code code code code...>"

	def handle(self, *args, **options):
		print("Querying database at %s" % (CURRENCY_API_URL))

		currencies = []
		if len(args):
			currencies = list(args)

		api = urlopen(CURRENCY_API_URL)
		d = json.loads(api.read())
		i = 0

		for currency in sorted(d.keys()):
			if (not currencies) or currency in currencies:
				if not Currency.objects.filter(code=currency):
					print("Creating %r (%s)" % (d[currency], currency))
					Currency(code=currency, name=d[currency], factor=1.0, is_active=False).save()
					i+=1

		if i == 1:
			print("%i new currency" % (i))
		else:
			print("%i new currencies" % (i))


from django import template
from django.template.defaultfilters import stringfilter
from currencies.models import Currency
from currencies.utils import calculate_price

register = template.Library()


@register.filter(name='currency')
@stringfilter
def set_currency(value, arg):
    return calculate_price(value, arg)


class ChangeCurrencyNode(template.Node):

    def __init__(self, price, currency):
        self.price = template.Variable(price)
        self.currency = template.Variable(currency)

    def render(self, context):
        try:
            return calculate_price(self.price.resolve(context),
                self.currency.resolve(context))
        except template.VariableDoesNotExist:
            return ''


@register.tag(name='change_currency')
def change_currency(parser, token):
    try:
        tag_name, current_price, new_currency = token.split_contents()
    except ValueError:
        tag_name = token.contents.split()[0]
        raise template.TemplateSyntaxError('%r tag requires exactly two arguments' % (tag_name))
    return ChangeCurrencyNode(current_price, new_currency)



from decimal import *
from django.test import TestCase
from django.conf import settings
from django import template
from currencies.models import Currency
from currencies.utils import calculate_price


class UtilsTest(TestCase):
    fixtures = ['test_data.json']
    use_transaction = False

    def test_calculate_price_success(self):
        res = calculate_price('10', 'USD')
        self.assertEqual(res, Decimal('15.00'))

    def test_calculate_price_fail(self):
        res = calculate_price('10', 'GBP')
        self.assertEqual(res, Decimal('0.00'))

    def test_calculate_price_currency_doesnotexist(self):
        settings.DEBUG = True
        self.assertRaises(Currency.DoesNotExist, calculate_price, '10', 'GBP')
        settings.DEBUG = False


class TemplateTagTest(TestCase):
    fixtures = ['test_data.json']
    use_transaction = False

    def setUp(self):
        self.template = '{% load currency %}'

    def test_currency_filter(self):
        t = template.Template(self.template +
            '{{ 10|currency:"USD" }}'
        )
        c = template.Context()
        s = t.render(c)
        self.assertEqual(s, u'15.00')

    def test_change_currency_tag_success(self):
        t = template.Template(self.template +
            '{% change_currency 10 "USD" %}'
        )
        c = template.Context()
        s = t.render(c)
        self.assertEqual(s, u'15.00')

    def test_change_currency_tag_fail(self):
        t = template.Template(self.template +
            '{% change_currency 10 "GPB" %}'
        )
        c = template.Context()
        s = t.render(c)
        self.assertEqual(s, u'0.00')



from django.db import models
from django.utils.translation import gettext_lazy as _


class Currency(models.Model):
    code = models.CharField(_('code'), max_length=3)
    name = models.CharField(_('name'), max_length=35)
    symbol = models.CharField(_('symbol'), max_length=4, blank=True)
    factor = models.DecimalField(_('factor'), max_digits=10, decimal_places=4,
        help_text=_('Specifies the difference of the currency to default one.'))
    is_active = models.BooleanField(_('active'), default=True,
        help_text=_('The currency will be available.'))
    is_base = models.BooleanField(_('base'), default=False,
        help_text=_('Make this the base currency against which rates are calculated.'))
    is_default = models.BooleanField(_('default'), default=False,
        help_text=_('Make this the default user currency.'))

    class Meta:
        ordering = ('name', )
        verbose_name = _('currency')
        verbose_name_plural = _('currencies')

    def __unicode__(self):
        return self.code

    def save(self, **kwargs):
        # Make sure the base and default currencies are unique
        if self.is_base:
            Currency.objects.filter(is_base=True).update(is_base=False)
        if self.is_default:
            Currency.objects.filter(is_default=True).update(is_default=False)
        super(Currency, self).save(**kwargs)



from decimal import Decimal, ROUND_UP
from currencies.models import Currency


def calculate_price(price, currency):
    price = Decimal(price)
    currency = Currency.objects.get(code__exact=currency)
    default = Currency.objects.get(is_default=True)

    # First, convert from the default currency to the base currency
    price = price / default.factor

    # Now, convert from the base to the given currency
    price = price * currency.factor

    return price.quantize(Decimal("0.01"), rounding=ROUND_UP)



import json
import os
from django.core.management.base import NoArgsCommand
from currencies.models import Currency


class Command(NoArgsCommand):
	help = "Update the currency symbols for currently installed currencies"
	def handle_noargs(self, **options):
		rel_path = "./currencies.json"
		abs_file_path = os.path.join(os.path.dirname(__file__), rel_path)
		mappings = {}

		with open(abs_file_path) as df:
			mappings = json.load(df)
		i = 0
		for currency in Currency.objects.all():
			if not currency.symbol:
				symbol = mappings.get(currency.code)
				if symbol:
					currency.symbol = symbol
					currency.save()
					i+=1

		if i == 1:
			print("Updated %i symbol" % (i))
		else:
			print("Updated %i symbols" % (i))



from django.contrib import admin
from currencies.models import Currency


class CurrencyAdmin(admin.ModelAdmin):
    list_display = ("name", "is_active", "is_base", "is_default", "code", "symbol", "factor")
    list_filter = ("is_active", )
    search_fields = ("name", "code")

admin.site.register(Currency, CurrencyAdmin)



#!/usr/bin/env python
from distutils.core import setup
import os
from currencies import __version__


# Compile the list of packages available, because distutils doesn't have
# an easy way to do this.
packages, data_files = [], []
root_dir = os.path.dirname(__file__)
if root_dir:
    os.chdir(root_dir)

for dirpath, dirnames, filenames in os.walk('currencies'):
    # Ignore dirnames that start with '.'
    for i, dirname in enumerate(dirnames):
        if dirname.startswith('.'): del dirnames[i]
    if '__init__.py' in filenames:
        pkg = dirpath.replace(os.path.sep, '.')
        if os.path.altsep:
            pkg = pkg.replace(os.path.altsep, '.')
        packages.append(pkg)
    elif filenames:
        prefix = dirpath[11:] # Strip "currencies/" or "currencies\"
        for f in filenames:
            data_files.append(os.path.join(prefix, f))

data_files.append("management/commands/currencies.json")

setup(name='django-currencies',
    version=__version__,
    description='Adds support for multiple currencies as a Django application.',
    long_description=open('README.md').read(),
    author='Panos Laganakos',
    author_email='panos.laganakos@gmail.com',
    url='https://launchpad.net/django-currencies',
    package_dir={'currencies': 'currencies'},
    packages=packages,
    package_data={'currencies': data_files},
    classifiers=['Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Utilities'],
    )



from currencies.models import Currency


def currencies(request):
    currencies = Currency.objects.all()

    if not request.session.get('currency'):
        request.session['currency'] = Currency.objects.get(is_default__exact=True)

    return {
        'CURRENCIES': currencies,
        'CURRENCY': request.session['currency']
    }
